@@ -27,7 +27,7 @@ public:
      * Constructor
      */
     CPPHeaderCodeOperation (CPPHeaderCodeDocument * doc, UMLOperation * op,
-                             const QString & body = "", const QString & comment = "");
+                             const QString & body = QString(), const QString & comment = QString());
 
     /**
      * Empty Destructor
