@@ -19,8 +19,11 @@
 #include "debug_utils.h"
 #include "folder.h"
 #include "enum.h"
+#include "enumliteral.h"
+#include "messagewidget.h"
 #include "model_utils.h"
 #include "object_factory.h"
+#include "objectwidget.h"
 #include "operation.h"
 #include "package.h"
 #include "template.h"
@@ -29,6 +32,7 @@
 #include "umllistview.h"
 #include "umlobject.h"
 #include "umbrellosettings.h"
+#include "widget_factory.h"
 
 // kde includes
 #include <KMessageBox>
@@ -38,6 +42,12 @@
 #include <QMap>
 #include <QRegExp>
 
+#ifdef Q_OS_WIN
+#define PATH_SEPARATOR QLatin1Char(';')
+#else
+#define PATH_SEPARATOR QLatin1Char(':')
+#endif
+
 DEBUG_REGISTER_DISABLED(Import_Utils)
 #undef DBG_SRC
 #define DBG_SRC QLatin1String("Import_Utils")
@@ -517,9 +527,9 @@ UMLAttribute* addMethodParameter(UMLOperation *method,
 /**
  * Add an enum literal to an UMLEnum.
  */
-void addEnumLiteral(UMLEnum *enumType, const QString &literal, const QString &comment)
+void addEnumLiteral(UMLEnum *enumType, const QString &literal, const QString &comment, const QString &value)
 {
-    UMLObject *el = enumType->addEnumLiteral(literal);
+    UMLObject *el = enumType->addEnumLiteral(literal, Uml::ID::None, value);
     el->setDoc(comment);
 }
 
@@ -598,7 +608,7 @@ QStringList includePathList()
     QStringList includePathList(incPathList);
     QString umbrello_incpath = QString::fromLatin1(qgetenv("UMBRELLO_INCPATH"));
     if (!umbrello_incpath.isEmpty()) {
-        includePathList += umbrello_incpath.split(QLatin1Char(':'));
+        includePathList += umbrello_incpath.split(PATH_SEPARATOR);
     }
     return includePathList;
 }
@@ -622,4 +632,232 @@ bool isDatatype(const QString& name, UMLPackage *parentPkg)
     return (o!=NULL);
 }
 
+/**
+ * Helper function to parse stack trace line. The supported formats are:
+ *
+ * 1. gdb
+ * 2. Qtcreator/gdb
+ *
+ * @param s string to parse
+ * @param sequence return of sequence number
+ * @param package return of package
+ * @param method return of method
+ * @return true line could be parsed and return variable has been filled
+ * @return false line could not be parsed, no return variable has been filled
+ */
+bool parseStraceTraceLine(const QString &s, QString &sequence, QString &package, QString &method)
+{
+    QString identifier;
+    QString module;
+    QStringList cols = s.split(QRegExp(QLatin1String("\\s+")),QString::SkipEmptyParts);
+    if (cols.size() < 2) {
+        DEBUG(DBG_SRC) << "could not parse" << s;
+        return false;
+    }
+    // skip comments
+    if (cols[0] == QLatin1String("#")) {
+        return false;
+    }
+
+    /*  gdb
+     * #0  0x000000000050d0b0 in Import_Utils::importStackTrace(QString const&, UMLScene*) (fileName=..., scene=scene@entry=0x12bd0f0)
+     * #25 0x00007fffefe1670c in g_main_context_iteration () from /usr/lib64/libglib-2.0.so.0
+     * #0  Import_Utils::importStackTrace (fileName=..., scene=scene@entry=0x137c000) at /home/ralf.habacker/src/umbrello-3/umbrello/codeimport/import_utils.cpp:715
+     */
+    if (cols.size() > 2 && cols[0].startsWith(QLatin1String("#"))) {
+        QString file;
+        sequence = cols.takeFirst();
+        if (cols[cols.size()-2] == QLatin1String("at")) {
+            file = cols.takeLast();
+            cols.takeLast();
+        }
+        else if (cols[cols.size()-2] == QLatin1String("from")) {
+            module = cols.takeLast();
+            cols.takeLast();
+        }
+
+        if (cols[1] == QLatin1String("in")) {
+            cols.takeFirst(); // remove address
+            cols.takeFirst(); // in
+        }
+
+        identifier = cols.join(QLatin1String(" "));
+
+        if (identifier.contains(QLatin1String("::"))) {
+            QStringList b = identifier.split( QLatin1String("::"));
+            // TODO handle multiple '::'
+            package = b.takeFirst();
+            method = b.join(QLatin1String("::"));
+        }
+        else {
+            method = identifier;
+            package = module;
+        }
+        return true;
+    }
+
+    /**
+     * Qtcreator/gdb
+     *  6	Driver::ParseHelper::ParseHelper	driver.cpp	299	0x634c44
+     * 31   g_main_context_dispatch /usr/lib64/libglib-2.0.so.0     0x7fffefe16316
+     */
+    else if (cols[cols.size()-1].startsWith(QLatin1String("0x"))) {
+        if (cols[0] == QLatin1String("..."))
+            return false;
+
+        sequence = cols.takeFirst();
+        cols.takeLast();  // remove address
+
+        QString line, file;
+        if (cols.size() == 2) {
+            module = cols.takeLast();
+            identifier = cols.join(QLatin1String(" "));
+        } else {
+            line = cols.takeLast();
+            file = cols.takeLast();
+            identifier = cols.join(QLatin1String(" "));
+        }
+
+        if (identifier.contains(QLatin1String("::"))) {
+            QStringList b = identifier.split( QLatin1String("::"));
+            method = b.takeLast();
+            package = b.join(QLatin1String("::"));
+        }
+        else {
+            method = identifier;
+            package = module;
+        }
+
+        if (package.isEmpty() && !file.isEmpty())
+            package = file;
+
+        if (!method.endsWith(QLatin1String(")")))
+            method.append(QLatin1String("()"));
+
+        return true;
+    } else
+        return false;
+}
+
+/**
+ * Import stack trace generated by gdb or qtcreator running gdb.
+ *
+ * The stack trace can be generated by
+ *  1. copy and paste of the output of gdb 'bt' command into a file (single line for each stack frame)
+ *  2. select frames in qtcreator stack window, right click and select "copy into clipboard", then paste into file
+ *
+ * @param fileName filename to import the stack trace from.
+ * @param scene The diagram to import the stack trace into.
+ * @return true Import successful.
+ * @return false Import failed.
+ */
+bool importStackTrace(const QString &fileName, UMLScene *scene)
+{
+    QFile file(fileName);
+
+    if(!file.open(QIODevice::ReadOnly))
+        return false;
+
+    // reverse lines
+    QStringList lines;
+    QTextStream in(&file);
+    in.setCodec("UTF-8");
+    while (!in.atEnd()) {
+        lines.insert(0, in.readLine());
+    }
+
+    // object widget cache map
+    QMap<QString, ObjectWidget*> objectsMap;
+
+    // create "client widget"
+    UMLDoc *umldoc = UMLApp::app()->document();
+    QString name(QLatin1String("client"));
+    UMLObject *left = umldoc->findUMLObject(name, UMLObject::ot_Class);
+    if (!left ) {
+        left = new UMLObject(0, name);
+        left->setBaseType(UMLObject::ot_Class);
+    }
+
+    ObjectWidget *leftWidget = (ObjectWidget *)Widget_Factory::createWidget(scene, left);
+    leftWidget->activate();
+    // required to be savable
+    scene->widgetList().append(leftWidget);
+    objectsMap[name] = leftWidget;
+
+    ObjectWidget *rightWidget = 0;
+    ObjectWidget *mostRightWidget = leftWidget;
+    MessageWidget *messageWidget = 0;
+    // for further processing
+    MessageWidgetList messages;
+
+    // for each line
+    int index = 1;
+    foreach(const QString &line, lines) {
+        QString stackframe, package, method;
+
+        if (!parseStraceTraceLine(line, stackframe, package, method))
+            continue;
+
+        bool createObject = false;
+        if (package.contains(method))
+            createObject = true;
+
+        if (package.isEmpty())
+            package = QLatin1String("unknown");
+
+        // get or create right object widget
+        if (objectsMap.contains(package)) {
+            rightWidget = objectsMap[package];
+        } else {
+            UMLObject *right = umldoc->findUMLObject(package, UMLObject::ot_Class);
+            if (!right)
+                right = umldoc->findUMLObject(package, UMLObject::ot_Package);
+            if (!right) {
+                right = Object_Factory::createUMLObject(UMLObject::ot_Class, package);
+            }
+
+            rightWidget = (ObjectWidget *)Widget_Factory::createWidget(scene, right);
+            rightWidget->setX(mostRightWidget->x() + mostRightWidget->width() + 10);
+            rightWidget->activate();
+            objectsMap[package] = rightWidget;
+            scene->widgetList().append(rightWidget);
+            mostRightWidget = rightWidget;
+        }
+
+        // create message
+        int y = 10 + (messageWidget ? messageWidget->y() + messageWidget->height() : 20);
+        messageWidget = new MessageWidget(scene, leftWidget, rightWidget, y,
+                                          createObject ? Uml::SequenceMessage::Creation : Uml::SequenceMessage::Synchronous);
+        messageWidget->setCustomOpText(method);
+        messageWidget->setSequenceNumber(QString::number(index++));
+        messageWidget->calculateWidget();
+        messageWidget->activate();
+        messageWidget->setY(y);
+        // to make it savable
+        scene->messageList().append(messageWidget);
+        messages.insert(0, messageWidget);
+
+        leftWidget = rightWidget;
+    }
+
+    if (messages.isEmpty())
+        return false;
+
+    // adjust vertical position
+    foreach(MessageWidget *w, messages) {
+        w->setY(w->y() + 20);
+    }
+
+    // adjust heights starting from the last message
+    MessageWidget *previous = messages.takeFirst();
+    foreach(MessageWidget *w, messages) {
+        w->setSize(w->width(), previous->y() - w->y() + previous->height() + 5);
+        // adjust vertical line length of object widgets
+        w->objectWidget(Uml::RoleType::A)->slotMessageMoved();
+        w->objectWidget(Uml::RoleType::B)->slotMessageMoved();
+        previous = w;
+    }
+    return true;
+}
+
 }  // end namespace Import_Utils
