@@ -195,11 +195,11 @@ void RefactoringAssistant::setVisibilityIcon(QTreeWidgetItem *item, const UMLObj
  * If no parameter is given the sender is used.
  * @param obj   the UML object
  */
-void RefactoringAssistant::objectModified(const UMLObject *obj)
+void RefactoringAssistant::objectModified()
 {
-    if (!obj) {
-        obj = dynamic_cast<const UMLObject*>(sender());
-    }
+    const UMLObject *obj = dynamic_cast<const UMLObject*>(sender());
+    if (!obj)
+        return;
     QTreeWidgetItem *item = findListViewItem(obj);
     if (!item) {
         return;
@@ -421,12 +421,12 @@ void RefactoringAssistant::deleteItem(QTreeWidgetItem *item, UMLObject *obj)
  */
 QAction* RefactoringAssistant::createAction(const QString& text, const char * method, const Icon_Utils::IconType icon)
 {
+    Q_UNUSED(method);
     QAction* action = new QAction(this);
     action->setText(text);
     if (icon != Icon_Utils::N_ICONTYPES) {
         action->setIcon(Icon_Utils::SmallIcon(icon));
     }
-    connect(action, SIGNAL(triggered()), this, method);
     return action;
 }
 
